<?php

namespace VerifoneEcomAPI\ApiWrapper\Http;

use Exception;
use VerifoneEcomAPI\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomAPI\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomAPI\ApiWrapper\Regions\Emea;
use VerifoneEcomAPI\ApiWrapper\Regions\Na;
use VerifoneEcomAPI\ApiWrapper\Regions\NzAu;
use VerifoneEcomAPI\ApiWrapper\Regions\Au;
use VerifoneEcomAPI\ApiWrapper\Settings;
use WC_Logger;

class AutoSetupService {
    private array $cardList = [ 'AMEX', 'CB', 'VPAY', 'ISRAEL_PRIVATE_LABEL', 'ISRACARD', 'Q_CARD', 'CARNET', 'DINERS', 'DISCOVER', 'ELECTRON', 'JCB', 'MAESTRO', 'UPI', 'VISA', 'MASTERCARD' ];
    private Client $client;
    private Settings $settings;
    private BasicAuth $auth;
    private string $userId;
    private string $apiKey;
    private bool $test;
    private $region;

    /**
     * @var WC_Logger
     */
    protected $logger;

    public function __construct( array $params, Settings $settings ) {
        $this->settings = $settings;
        $this->userId   = $params['as_user_id'];
        $this->apiKey   = $params['as_api_key'];
        $this->test     = $params['env'] !== 'production';
        $this->region   = $params['region'];

        $this->settings->setApiUserId( $this->userId )
                       ->setApiKey( $this->apiKey )
                       ->setTest( $this->test );
        $this->auth = new BasicAuth( $this->settings );
        switch ( $params['region'] ) {
            case 'NZAU':
                $objRegion = new NzAu( $this->settings );
                break;
            case 'AU':
                $objRegion = new Au( $this->settings );
                break;
            case 'NA':
                $objRegion = new Na( $this->settings );
                break;
            default: // 'EMEA'
                $objRegion = new Emea( $this->settings );
                break;
        }

        $this->logger = new \WC_Logger();
        $this->client = new Client($objRegion, $this->settings, $this->auth, new SimpleCurl());
    }

    /**
     *  List all entities for this user_id-api_key pair
     *  User must have access only to one Merchant Company and 1 Merchant Site org, or just 1 Merchant Site org
     *  if there are more Merchant Company or Site org, we return an error
     * @return false|mixed
     * @throws HttpException
     */
    private function getEntities() {
        $entityList = $this->client->getEntities();
        $list       = [];
        foreach ( $entityList as $entity ) {
            if ( $entity['entityType'] === 'MERCHANT_SITE' ) {
                $list[] = $entity;
            }
        }

        return count( $list ) === 1 ? $list[0] : false;
    }


    /**
     * @throws HttpException
     */
    private function getPPCs( $entityId ) {
        return $this->client->getPPCs( $entityId );
    }

    /**
     * User only has 1 PPC for Cards (might be separate PPCs for Gift Cards and PLCC)
     *
     * @param $ppcList
     *
     * @return array|bool|float|int|object|string|null
     */
    private function getPPC4card( $ppcList ) {
        $key      = 0;
        $validPPC = null;
        foreach ( $ppcList as $ppc ) { // iterate through all PPCs
            foreach ( $ppc['paymentType'] as $pt ) { // we look in all payment types
                if ( in_array( $pt, $this->cardList ) && in_array( 'ECOMMERCE', $ppc['salesChannels'] ) ) { // check if it's a valid card name (not like PayPal or something else)
                    $key ++;
                    $validPPC = $ppc;
                    break;
                }
            }
        }

        return $key === 1 ? $validPPC : null;
    }


    /**
     * User only has one PPC for Paypal
     *
     * @param $ppcList
     *
     * @return array|bool|float|int|object|string|null
     */
    private function getPaypalConfig( $ppcList ) {
        $key      = 0;
        $validPPC = null;
        foreach ( $ppcList as $ppc ) { // iterate through all PPCs
            foreach ( $ppc['paymentType'] as $pt ) { // we look in all payment types
                if ( in_array( $pt, [ 'PAYPAL_ECOM', 'PAYPAL_ECOM_MANAGED' ] ) && in_array( 'ECOMMERCE', $ppc['salesChannels'] ) ) {
                    $key ++;
                    $validPPC = $ppc;
                    break;
                }
            }
        }

        return $key === 1 ? $validPPC : null;
    }

    private function getBankOpConfig($ppcList) {
        $key      = 0;
        $validPPC = null;

        foreach ($ppcList as $ppc) {
            foreach ($ppc['paymentType'] as $pt) {
                if (in_array($pt, ['OP_ONLINE_PAYMENT']) && in_array('ECOMMERCE', $ppc['salesChannels'])) {
                    return $ppc;
                }
            }
        }

        return null;
    }

    /**
     * @throws HttpException
     */
    private function get3DS() {
        $threeDs = $this->client->get3DS();

        return count( $threeDs ) ? $threeDs[0] : false;
    }


    /**
     * @throws HttpException
     */
    private function getWallets() {
        return $this->client->getWallets();
    }


    /**
     * @throws HttpException
     */
    public function setAccount(): array {

        if ( ! $this->client->checkCredentials() ) {
            return [
                'status'  => false,
                'message' => 'Invalid credentials. Make sure you enter right credentials/environment and try again!'
            ];
        }

        //default response
        $response = [
            'status'  => false,
            'message' => 'Account could not be auto-configured, manual configuration is required'
        ];
        $entity   = $this->getEntities();
        if ( ! $entity ) {
            return $response;
        }

        $ppcList   = $this->getPPCs( $entity['entityUid'] );
        $paypalPpc = $this->getPaypalConfig( $ppcList );
        $bankOkPpc = $this->getBankOpConfig($ppcList);
        $ppc       = $this->getPPC4card( $ppcList );
        if ( ! $ppc && ! $paypalPpc ) {
            return $response;
        }
        $config = [
            'title'                     => 'Verifone Hosted Cart',
            'description'               => 'Payments powered by <a href="https://www.verifone.com/" target="_blank">Verifone</a>',
            'enabled'                   => 'yes',
            'complete_order_on_payment' => 'yes',
            'curl_verify_ssl'           => 'yes',
            'fraud_check_enabled'       => 'no',
            'subscription_enabled'      => 'no',
            'transaction_type'          => 'sale',
            'autoSetup'                 => true,
            'payment_type'              => 'iframe',
            'checkout_button_text'      => 'Place order',
            'test'                      => $this->test ? 'yes' : 'no',
            'user_id'                   => $this->userId,
            'api_key'                   => $this->apiKey,
            'entity_id'                 => $entity['entityUid'],
            'region'                    => $this->region
        ];

        if ( $paypalPpc ) {
            $config['paypal_enabled']     = 'yes';
            $config['paypal_contract_id'] = $paypalPpc['contractUid'];
        }

        if ($bankOkPpc) {
            $config['bank_op_enabled'] = 'yes';
            $config['bank_op_contract_id'] = $bankOkPpc['contractUid'];
        }

        // card 3ds
        $threeDs = $this->get3DS();
        if ( $threeDs ) {
            $config['_3ds_enable']      = 'yes';
            $config['_3ds_contract_id'] = $threeDs['contractUid'];
        }

        //card
        if ( $ppc && array_diff( $this->cardList, $ppc['paymentType'] ) ) {
            $config['card_enabled']        = 'yes';
            $config['payment_contract_id'] = $ppc['contractUid'];

            //wallets
            $wallets = $this->getWallets();
            foreach ( $wallets as $wallet ) {
                if ( $wallet['contractType'] === 'VIPPS_ECOM' ) { //vipps
                    $config['vipps_enabled']              = 'yes';
                    $config['vipps_contract_id']          = $ppc['contractUid'];
                    $config['vipps_sca_compliance_level'] = $threeDs ? 'WALLET' : 'NONE';
                    if ( $threeDs ) {
                        $config['vipps_threeds_contract_id'] = $threeDs['contractUid'];
                    }
                }
                if ( $wallet['contractType'] === 'MOBILEPAY_ECOM' ) { //mobile pay
                    $config['mobile_pay_enabled']              = 'yes';
                    $config['mobile_pay_contract_id']          = $ppc['contractUid'];
                    $config['mobile_pay_sca_compliance_level'] = $threeDs ? 'WALLET' : 'NONE';
                    if ( $threeDs ) {
                        $config['mobile_pay_threeds_contract_id'] = $threeDs['contractUid'];
                    }
                }
                if ( $wallet['contractType'] === 'APPLE_PAY_WEB' ) { //apple pay
                    $config['apple_pay_enabled']              = 'yes';
                    $config['apple_pay_contract_id']          = $ppc['contractUid'];
                    $config['apple_pay_sca_compliance_level'] = $threeDs ? 'WALLET' : 'NONE';
                    if ( $threeDs ) {
                        $config['apple_pay_3ds_enable']          = 'yes';
                        $config['apple_pay_threeds_contract_id'] = $threeDs['contractUid'];
                    }
                }

                if ( $wallet['contractType'] === 'GOOGLE_PAY' ) { //google pay
                    $config['google_pay_enabled']     = 'yes';
                    $config['google_pay_contract_id'] = $ppc['contractUid'];
                    $config['google_pay_sca_compliance_level'] = $threeDs ? 'WALLET' : 'NONE';
                    if ( $threeDs ) {
                        $config['google_pay_3ds_enable']          = 'yes';
                        $config['google_pay_threeds_contract_id'] = $threeDs['contractUid'];
                    }
                }
            }
        }

        foreach ( $ppcList as $item ) {
            //klarna
            if ( in_array( 'KLARNA_ECOM', $item['paymentType'] ) ) {
                $config['klarna_enabled'] = 'yes';
            }
            //swish
            if ( in_array( 'SWISH', $item['paymentType'] ) ) {
                $config['swish_pay_enabled'] = 'yes';
            }
            //gift card
            if ( in_array( 'GIFT_CARD', $item['paymentType'] ) ) {
                $config['gift_card_enabled']     = 'yes';
                $config['gift_card_contract_id'] = $item['contractUid'];
            }
            //plcc
            if ( in_array( 'PLCC', $item['paymentType'] ) ) {
                $config['plcc_enabled']     = 'yes';
                $config['plcc_contract_id'] = $item['contractUid'];
            }
        }

        $this->webhookSetup($entity['entityUid']);

        // first, we empty the config field
        update_option( 'woocommerce_verifone_hosted_cart_settings', [] );
        // create/update entry in DB
        update_option( 'woocommerce_verifone_hosted_cart_settings', $config ); // then we populate it with new data

        return [
            'status'  => true,
            'message' => 'Your account has been successfully configured!',
            'config'  => $config
        ];
    }


    public function webhookSetup( $entityId ): void {
        $url = WC()->api_request_url( 'woo-verifone-webhook' );
        $webhooks = $this->client->getWebhooks();

        // if we find a hook that matches our url, we stop
        foreach ($webhooks['triggers'] as $trigger) {
            foreach ($trigger['endpoints'] as $endpoint) {
                if ($endpoint['url'] === $url) {
                    return;
                }
            }
        }

        //save new webhook
       $this->client->setWebhook( $entityId, $url );
    }

}
