<?php

use VerifoneEcomAPI\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomAPI\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomAPI\ApiWrapper\Regions\Emea;
use VerifoneEcomAPI\ApiWrapper\Regions\Na;
use VerifoneEcomAPI\ApiWrapper\Regions\NzAu;
use VerifoneEcomAPI\ApiWrapper\Settings;

if ( ! defined( 'ABSPATH' ) ) {
    exit;
} // Exit if accessed directly

// if the WC payment gateway class is not available, do nothing
if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
    return;
}

/**
 * Class WC_Verifone
 */
abstract class WC_Verifone extends \WC_Payment_Gateway {
    static $REFUND_BUTTON_RENDERED = false;
    protected $plugin_name;
    protected $version;
    protected $method_long_description;
    protected $method_description_link_text;
    protected $method_description_link_url;
    const NORWAY_CODES = [ 'nb', 'nn' ];

    protected $refundStatusOk = [
        VERIFONE_TRANSACTION_AUTHORIZED,
        VERIFONE_TRANSACTION_AUTHORISED,
        VERIFONE_TRANSACTION_INITIATED,
        VERIFONE_TRANSACTION_REFUNDED,
        VERIFONE_TRANSACTION_VOIDED,
        VERIFONE_TRANSACTION_REFUND_SETTLEMENT_COMPLETED,
        VERIFONE_TRANSACTION_VOID_SETTLED,
    ];
    protected $client;

    /**
     * @var BasicAuth
     */
    protected $verifone_auth_helper;

    /**
     * @var Settings
     */
    protected $verifone_settings;

    /**
     * @var WC_Logger
     */
    protected $logger;

    /**
     * @var Emea|Na|NzAu
     */
    protected $api_url_mapper;

    /**
     * @var SimpleCurl
     */
    protected $simple_curl;

    static $PLUGIN_ID;

    /**
     * WC_Verifone_Hosted_Cart constructor.
     */
    public function __construct() {
        //Initialize all the basics components of the plugin
        load_plugin_textdomain( 'verifone-hosted', false, 'verifone-hosted/languages' );
        $this->plugin_name                  = 'Verifone Hosted Cart Payment';
        $this->method_description           = __( 'Card payments and alternative payment methods powered by Verifone', 'verifone-hosted' );
        $this->method_long_description      = __( 'Accept card payments and other payment options available in your Verifone account. Hosted cart or iFrame experiences available.', 'verifone-hosted' );
        $this->method_description_link_text = __( 'Read the documentation.', 'verifone-hosted' );
        $this->method_description_link_url  = 'https://verifone.cloud/docs/online-payments/plugins/woocommerce';
        $this->supports                     = [
            'refunds',
            'subscriptions',
            'subscription_cancellation',
            'subscription_suspension',
            'subscription_reactivation',
            'subscription_amount_changes',
            'subscription_date_changes',
            'subscription_payment_method_change_customer',
        ];
        $this->method_title                 = __( 'Verifone Hosted Cart', 'verifone-hosted' );
        $this->version                      = '1.0.1';
        $this->id                           = $this->getPluginId();
        $this->enabled                      = $this->get_option( 'enabled' );
        $this->has_fields                   = false;
        // Define user set variables
        $this->title       = $this->get_option( 'title' );
        $this->description = $this->get_option( 'description', __( 'Payments powered by', 'verifone-hosted' ) . ' <a href="https://www.verifone.com/" target="_blank">Verifone</a>' );

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        $this->logger      = new WC_Logger();
        $this->simple_curl = new SimpleCurl();
    }

    public function api_response( $message = null, $statusCode = 200 ) {
        status_header( $statusCode );

        if ( is_array( $message ) ) {
            header( "Content-Type: application/json" );
            echo wp_json_encode( $message );
        } else {
            echo $message;
        }

        exit;
    }

    public function api_response_ok( $message = null ) {
        $this->api_response( [ 'info' => $message ], 200 );
    }

    public function api_response_error( $message = null, $code = null ) {
        $this->api_response( [ 'error' => $message, 'code' => $code ], 200 );
    }

    /**
     * @return array
     */
    protected function failure( ?string $message = null ) {
        if ( is_null( $message ) ) {
            $message = __( 'There has been an error processing your order', 'woocommerce-gateway-verifone-hosted-checkout' );
        }

        return [
            'result'   => 'failure',
            'messages' => $message,
        ];
    }

    public function trueOrFalse( $optionName ) {
        return 'yes' == strtolower( $this->get_option( $optionName ) );
    }

    /** Check if order has only downloadable products
     *
     * @param WC_Order $order
     *
     * @return bool
     */
    public function isOrderOnlyVirtual( WC_Order $order ) {
        foreach ( $order->get_items() as $item ) {
            $product = wc_get_product( $item->get_product_id() );
            if ( ! $product->is_virtual() ) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param $amount
     *
     * @return int
     */
    public function calculateAmount( $amount ): int {
        $_0decimals = [ 'XPF', 'XOF', 'XAF', 'VUV', 'VND', 'UYI', 'UGX', 'RWF', 'PYG', 'KRW', 'KMF', 'JPY', 'ISK', 'GNF', 'DJF', 'CLP', 'BIF' ];
        $_3decimals = [ 'TND', 'OMR', 'LYD', 'KWD', 'JOD', 'IQD', 'BHD' ];
        $_4decimals = [ 'UYW', 'CLF' ];
        $currency   = get_option( 'woocommerce_currency' );
        switch ( $currency ) {
            case in_array( $currency, $_0decimals ):
                $multiplier = 1;
                break;
            case in_array( $currency, $_3decimals ):
                $multiplier = 1000;
                break;
            case in_array( $currency, $_4decimals ):
                $multiplier = 10000;
                break;
            default: // for 2 decimals (vast majority)
                $multiplier = 100;
                break;
        }

        return absint( wc_format_decimal( ( $amount * $multiplier ), wc_get_price_decimals() ) );
    }

    protected function hasSubscriptions( array $event, WC_Order $order ) {
        if ( isset( $event['details'] ) &&
             isset( $event['details']['stored_credential'] ) &&
             isset( $event['details']['stored_credential']['reference'] ) &&
             function_exists( 'wcs_get_subscriptions_for_order' ) ) {

            if ( $order instanceof WC_Subscription ) {
                $subscriptions = [ $order ];
            } else {
                $subscriptions = wcs_get_subscriptions_for_order( $order->get_id() );
            }

            foreach ( $subscriptions as $subscription ) {
                $subscription->add_meta_data( VERIFONE_ORDER_STORED_CREDENTIAL_REFERENCE, $event['details']['stored_credential']['reference'], true );
                $subscription->save();
            }

            $order->add_order_note( __( "Subscription reference: " . $event['details']['stored_credential']['reference'] ) );
        }
    }

    /**
     * @param int $order_id
     * @param null $amount
     * @param string $reason
     *
     * @return bool|void|WP_Error
     */
    public function _process_refund( $order_id, $amount = null, $reason = '' ) {
        $order = wc_get_order( $order_id );

        if ( ! in_array( $order->get_payment_method(), VERIFONE_PAYMENT_METHODS ) ) {
            return;
        }

        // if the order was not captured and try to refund, we void the transaction
        if ( $order->get_status() === 'pending' &&
             in_array( $order->get_meta( VERIFONE_ORDER_TRANSACTION_TYPE ), [ 'FINAL_AUTH', 'PRE_AUTH' ] ) ) {

            $this->cancel_payment( $order->get_id() );

            return true;
        }

        $transaction_id = $order->get_meta( VERIFONE_ORDER_TRANSACTION_ID );
        $payment_method = $order->get_meta( VERIFONE_PAYMENT_METHOD_USED );
        if ( ! $transaction_id ) {
            $this->logger->debug( sprintf( 'Tried to refund order with order ID %s but it has no registered transaction ID, aborting.', $order_id ) );

            return new WP_Error( 'verifone_refund_error', __( 'Refund Error: Unable to refund transaction.' ) );
        }

        $date           = new DateTime( 'now' );
        $verifoneAmount = $this->calculateAmount( (float) $amount );

        if ( VERIFONE_PAYMENT_METHOD_PAYPAL === $payment_method ) {
            $refund = [
                'amount' => [
                    'value'        => $verifoneAmount,
                    'currencyCode' => $order->get_currency(),
                ]
            ];
        } else {
            $refund = [
                'amount' => $verifoneAmount,
            ];
        }

        if ( '' !== $reason ) {
            $refund['reason'] = $reason;
        }
        try {
            $refundType = 'refund';
            $response   = ( VERIFONE_PAYMENT_METHOD_PAYPAL === $payment_method )
                ? $this->client->postPaypalRefund( $refund, $transaction_id )
                : $this->client->postRefund( $refund, $transaction_id );
            $this->orderCheckAndLogEvent( $order, $response['id'] );
        } catch ( Exception $exception ) {
            $decoded   = json_decode( $exception->getMessage(), true );
            $orderData = $order->get_data_without_line_items();
            if ( isset( $decoded['details']['error'] ) &&
                 'this transaction cannot be refunded' == $decoded['details']['error'] &&
                 (string) $verifoneAmount == (string) $this->calculateAmount( $orderData['total'] )
            ) {
                try {
                    $refundType = 'void';
                    $response   = $this->client->postVoidAuthorization( [], $transaction_id );
                    $this->orderCheckAndLogEvent( $order, $response['id'] );
                } catch ( Exception $exception ) {
                    try {
                        $response = $this->client->postVoidCapture( $transaction_id );
                        $this->orderCheckAndLogEvent( $order, $response['id'] );
                    } catch ( Exception $exception ) {
                        $this->logger->error( 'There was an error in the API response: ' . PHP_EOL .
                                              'Received response: ' . $exception->getMessage() . PHP_EOL .
                                              'To endpoint: ' . $this->api_url_mapper->postVoidAuthorizationUrl( $transaction_id ) . PHP_EOL .
                                              'Line: ' . __LINE__
                        );

                        return new WP_Error( 'verifone_refund_error', __LINE__ . ': ' . __( 'Refund Error: Unable to refund transaction.' ) );
                    }
                }
            } else {
                $this->logger->error( 'There was an error in the API response: ' . PHP_EOL .
                                      'Sent request: ' . json_encode( $refund ) . PHP_EOL .
                                      'Received response: ' . $exception->getMessage() . PHP_EOL .
                                      'To endpoint: ' . $this->api_url_mapper->getRefundUrl( $transaction_id ) . PHP_EOL .
                                      'Line: ' . __LINE__
                );

                return new WP_Error( 'verifone_refund_error', __LINE__ . ': ' . __( 'Refund Error: Unable to refund transaction.' ) );
            }
        }

        if ( ! isset( $response['id'] )
             || ! isset( $response['status'] )
             || ! in_array( $response['status'], $this->refundStatusOk )
        ) {
            return new WP_Error( 'verifone_refund_error', __LINE__ . ': ' . __( 'Refund Error: Unable to refund transaction.' ) );
        }

        $order->update_meta_data( VERIFONE_ORDER_REFUND_ID, $response['id'] );
        $this->orderCheckAndLogEvent( $order, $response['id'] );

        if ( 'INITIATED' === $response['status'] ) {
            $order->add_order_note( sprintf( __( 'Refunded %s out of a total of %s was initiated, pending processing' ), $amount, $order->get_total() ) );
        } else {
            if ( 'void' === $refundType ) {
                $order->add_order_note( sprintf( __( 'Voided the authorization of %s from order' ), $amount, $order->get_total() ) );
            } else {
                $order->add_order_note( sprintf( __( 'Refunded %s out of a total of %s from order' ), $amount, $order->get_total() ) );
            }
        }
        $order->save_meta_data();
        $order->save();

        return true;
    }

    public function process_webhook() {
        $input = file_get_contents( 'php://input' );
        if ( ! $input ) {
            $this->api_response_error( 'Unable to process data' );
        }
        global $woocommerce;
        $data = json_decode( $input, true );
        if ( json_last_error() !== JSON_ERROR_NONE ) {
            $this->logger->error( __CLASS__ . '@' . __FUNCTION__ . ' line ' . __LINE__ . ': Unable to decode response from API' );
            $this->api_response_error( 'Unable to process data', __LINE__ );
        }

        if ( $this->stringStartsWith( $data['eventType'], 'Checkout' ) ) {
            try {
                $checkout         = $this->client->getCheckout( $data['recordId'] );
                $data['recordId'] = $checkout['transaction_id'];
            } catch ( Exception $exception ) {
                $this->logger->error( __CLASS__ . '@' . __FUNCTION__ . ' line ' . __LINE__ . ': There was a request for an order: ' . $exception->getMessage() );
                $this->api_response_error( 'Unable to process data', __LINE__ );
            }
        }

        try {
            $response = $this->client->getTransaction( $data['recordId'] );
        } catch ( Exception $exception ) {
            $this->logger->error( __CLASS__ . '@' . __FUNCTION__ . ' line ' . __LINE__ . ': There was a request for an order: ' . $exception->getMessage() );
            $this->api_response_error( 'Unable to retrieve transaction', __LINE__ );
        }

        $order = null;
        if ( ! empty( $response['merchant_reference'] ) ) {
            $order = wc_get_order( (int) $response['merchant_reference'] );
            if ( ! $order instanceof WC_Order ) {
                $this->logger->debug( __CLASS__ . '@' . __FUNCTION__ . ' line ' . __LINE__ . '. There was a request for an order (' . $response['merchant_reference'] . ') that doesn\'t exist in current shop! Record ID: ' . $data['recordId'] . ' , response: ' . json_encode( $response ) );
                $this->api_response_error( 'Unable to process data', __LINE__ );
            }
            $order->update_meta_data( VERIFONE_PAYMENT_METHOD_USED, $response['payment_product'] );
            $order->update_meta_data( VERIFONE_ORDER_TRANSACTION_ID, $data['recordId'] );
        } elseif ( ! empty( $response['id'] ) ) {
            $order = wc_get_orders( [
                'limit'        => 1,
                'orderby'      => 'date',
                'order'        => 'DESC',
                'meta_key'     => VERIFONE_ORDER_REFUND_ID,
                'meta_value'   => $response['id'],
                'meta_compare' => '=',
            ] );
            if ( $order ) {
                $order = $order[0];
            }
        }
        if ( ! $order instanceof WC_Order ) {
            $this->logger->debug( __CLASS__ . '@' . __FUNCTION__ . ' line ' . __LINE__ . '. There was a request for the Record ID: ' . $data['recordId'] . ' , response: ' . json_encode( $response ) );
            $this->api_response_error( 'Unable to process data', __LINE__ );
        }

        if ( $this->orderCheckAndLogEvent( $order, $data['eventId'] ) ) {
            $this->api_response_ok();
        }

        $this->logOrderComment( $data['eventType'], $data['recordId'], $order );
        switch ( $response['status'] ) {
            case VERIFONE_TRANSACTION_CAPTURE_AUTHORISED:
            case VERIFONE_TRANSACTION_CAPTURE_SETTLED:
            case VERIFONE_TRANSACTION_AUTHORISED:
            case VERIFONE_TRANSACTION_SALE_SETTLED:
            case VERIFONE_TRANSACTION_CARD_VERIFICATION_AUTHORISED:
                $order->payment_complete();
                if ( $this->verifone_settings->getCompleteOrderOnPayment() || $this->isOrderOnlyVirtual( $order ) ) {
                    $order->update_status( 'completed' );
                    $woocommerce->cart->empty_cart();
                }
                $order->add_order_note( __( "Order payment is completed." ), false );
                $this->hasSubscriptions( $data, $order );
                break;

            case VERIFONE_TRANSACTION_AUTHORISATION_AUTHORISED:
            case VERIFONE_TRANSACTION_SALE_AUTHORISED:
                if ( 'processing' !== $order->get_status() ) {
                    $order->add_order_note( __( 'The payment is not settled yet. 
                        Attempting to refund the order in full, before the order is settled, will void the transaction and cancel the payment.
                        Order status changed to Pending.' ), false );

                    if ( 'completed' !== $order->get_status() ) {
                        $order->update_status( 'processing' );
                    }
                }
                break;

            case VERIFONE_TRANSACTION_PENDING:
            case VERIFONE_TRANSACTION_PREAUTH_AUTHORISED:
                if ( 'pending' != $order->get_status()  && !$this->noteExists($order->get_id(),'(transaction is in PRE/FINAL AUTH status)')) {
                    $order->add_order_note( __( 'The payment is not settled yet (transaction is in PRE/FINAL AUTH status). 
                    Attempting to make a full refund, before capturing, will void the authorization and cancel the payment
                    Order status changed to Pending.' ), false );
                }
                $order->update_status( 'processing' );
                break;

            case VERIFONE_TRANSACTION_REFUND_AUTHORISED:
            case VERIFONE_TRANSACTION_REFUND_REFUNDED:
            case VERIFONE_TRANSACTION_REFUNDED:
            case VERIFONE_TRANSACTION_REFUND_SETTLED:
                $order->add_order_note( sprintf( __( "Refunded %s out of a total of %s from order." ), $response['amount'], $order->get_total() ), false );
                wc_create_refund( [
                    'order_id'       => $order->get_id(),
                    'line_items'     => [],
                    'refund_payment' => false,
                    'amount'         => $response['amount'],
                ] );
                break;

            case VERIFONE_TRANSACTION_VOIDED:
            case VERIFONE_TRANSACTION_VOID_SETTLED:
            case VERIFONE_TRANSACTION_VOID_AUTHORISED:
                $order->add_order_note( sprintf( __( 'Voided the authorization of %s from order' ), $response['amount'] ), false );
                wc_create_refund( [
                    'order_id'       => $order->get_id(),
                    'line_items'     => [],
                    'refund_payment' => false,
                    'amount'         => $response['amount'],
                ] );
                break;

            default:
                if ( in_array( $data['eventType'], VERIFONE_WEBHOOK_EVENT_DECLINED ) ) {
                    $order->add_order_note( __( "Payment was declined" ), false );
                    $order->update_status( 'failed' );
                } elseif ( VERIFONE_WEBHOOK_EVENT_TYPE_REFUND_DECLINED === $data['eventType'] ) {
                    $verifoneResponseAmount = (float) $response['amount'];

                    $refunds = $order->get_refunds();
                    foreach ( $refunds as $refund ) {
                        $refundAmount = (float) $refund->get_amount();
                        if ( 'completed' === $refund->get_status() && $verifoneResponseAmount === $refundAmount ) {
                            $refund->delete();
                            break;
                        }
                    }
                }
        }

        $this->api_response_ok();
    }

    abstract function get_webhook_response_url();

    abstract function getPluginId(): string;

    public function generate_radio_html( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = array(
            'title'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'placeholder'       => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => array(),
        );

        $data = wp_parse_args( $data, $defaults );

        $defaultValue = isset($data['default']) ? $data['default'] : null;

        ob_start();
        ?>
        <tr style="vertical-align: top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?><?php echo $this->get_tooltip_html( $data ); ?></label>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
                    <?php
                    foreach ( $data["options"] as $optionValue => $optionLabel ) {
                        ?>
                        <input id="<?php echo esc_attr( $field_key ); ?>" type="radio" name="<?php echo esc_attr( $field_key ); ?>" value="<?php echo $optionValue ?>" <?php checked( $optionValue,
                            $this->get_option( $key, $defaultValue ), true ); ?>> <?php echo $optionLabel ?>
                        <?php
                    }
                    echo $this->get_description_html( $data );
                    ?>
                </fieldset>
            </td>
        </tr>
        <?php

        return ob_get_clean();
    }

    public function generate_checkbox_html( $key, $data ) {
        $field_key = $this->get_field_key( $key );
        $defaults  = array(
            'title'             => '',
            'label'             => '',
            'disabled'          => false,
            'class'             => '',
            'css'               => '',
            'type'              => 'text',
            'desc_tip'          => false,
            'description'       => '',
            'custom_attributes' => array(),
        );

        $data = wp_parse_args( $data, $defaults );

        if ( ! $data['label'] ) {
            $data['label'] = $data['title'];
        }
        $isChecked = $this->get_option( $key );

        // we force 3ds for NZAU
        if ( $field_key === 'woocommerce_verifone_hosted_cart__3ds_enable' &&
             $this->verifone_settings->getRegion() === 'NZAU' ) {
            $isChecked        = 'yes';
            $data['disabled'] = true;
        }
        ob_start();
        ?>
        <tr style="vertical-align: top">
            <th>
                <fieldset>
                    <legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
                    <label for="<?php echo esc_attr( $field_key ); ?>" style="white-space: nowrap">
                        <input <?php disabled( $data['disabled'], true ); ?> class="<?php echo esc_attr( $data['class'] ); ?>" type="checkbox" name="<?php echo esc_attr( $field_key ); ?>"
                                                                             id="<?php echo esc_attr( $field_key ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>"
                                                                             value="1" <?php checked( $isChecked, 'yes' ); ?>
                            <?php echo $this->get_custom_attribute_html( $data ); ?> /> <?php echo wp_kses_post( $data['label'] ); ?>
                        <?php echo $this->get_tooltip_html( $data ); ?>
                    </label>
                    <?php echo $this->get_description_html( $data ); ?>
                </fieldset>
            </th>
            <td></td>
        </tr>
        <?php

        return ob_get_clean();
    }

    /**
     * substrings an string
     *
     * @param $string
     * @param int $length
     *
     * @return false|string
     */
    protected function s40( $string, $length = 40 ) {

        return substr( $string, 0, $length );
    }

    protected function sanitizeValue( &$value, $replace = ',' ) {
        $charset = [ '\\', '/', ':', ';', '\'', '-' ];
        $value   = str_replace( $charset, $replace, $value );
    }

    protected function sanitizeCustomerInfo( &$customerInfo ) {
        foreach ( array_keys( $customerInfo ) as $key ) {
            $replace = ',';

            if ( in_array( $key, [ 'phone', 'postal_code' ] ) ) {
                $replace = '';
            }

            $this->sanitizeValue( $customerInfo[ $key ], $replace );
        }
    }

    /**
     * Creates the customer object returns the customer_id
     *
     * @param $order
     *
     * @return mixed|null
     */
    protected function getCustomer( $order ) {
        try {

            $customer_info = [];
            $phone         = preg_replace( '/[^0-9]+/', '', $this->s40( $order->get_billing_phone(), 20 ) );

            $customer_info['entity_id'] = $this->verifone_settings->getEntityId();

            if ( ! empty( $phone ) ) {
                $customer_info['phone_number']     = $phone;
                $customer_info['billing']['phone'] = $phone;
            }
            //billing
            if ( ! empty( $order->get_billing_company() ) ) {
                $customer_info['company_name'] = $this->s40( $order->get_billing_company(), 100 );
            }
            if ( ! empty( $order->get_billing_email() ) ) {
                $customer_info['email_address'] = $this->s40( $order->get_billing_email(), 255 );
            }
            if ( ! empty( $order->get_billing_address_1() ) ) {
                $customer_info['billing']['address_1'] = $this->s40( $order->get_billing_address_1() );
            }
            if ( ! empty( $order->get_billing_city() ) ) {
                $customer_info['billing']['city'] = $this->s40( $order->get_billing_city() );
            }
            if ( ! empty( $order->get_billing_country() ) ) {
                $customer_info['billing']['country_code'] = strtoupper( $order->get_billing_country() );
            } else {
                $customer_info['billing']['country_code'] = strtoupper( WC()->countries->get_base_country() );
            }
            if ( ! empty( $order->get_billing_first_name() ) ) {
                $customer_info['billing']['first_name'] = $this->s40( $order->get_billing_first_name(), 50 );
            }
            if ( ! empty( $order->get_billing_last_name() ) ) {
                $customer_info['billing']['last_name'] = $this->s40( $order->get_billing_last_name(), 50 );
            }
            if ( ! empty( $order->get_billing_postcode() ) ) {
                $customer_info['billing']['postal_code'] = $this->s40( $order->get_billing_postcode(), 10 );
            }
            if ( ! empty( $order->get_billing_state() ) ) {
                $customer_info['billing']['state'] = $this->s40( $order->get_billing_state(), 50 );
            }
            if ( ! empty( $order->get_billing_address_2() ) ) {
                $customer_info['billing']['address_2'] = $this->s40( $order->get_billing_address_2() );
            }

            //shipping
            if ( '' === $order->get_shipping_address_1() ) {
                $customer_info['shipping'] = $customer_info['billing'];
            } else {
                $phoneShipping = preg_replace( '/[^0-9]+/', '', $this->s40( $order->get_shipping_phone(), 20 ) );
                if ( ! empty( $phoneShipping ) ) {
                    $customer_info['shipping']['phone'] = $phoneShipping;
                }
                if ( ! empty( $order->get_shipping_address_1() ) ) {
                    $customer_info['shipping']['address_1'] = $this->s40( $order->get_shipping_address_1() );
                }
                if ( ! empty( $order->get_shipping_city() ) ) {
                    $customer_info['shipping']['city'] = $this->s40( $order->get_shipping_city() );
                }
                if ( ! empty( $order->get_shipping_country() ) ) {
                    $customer_info['shipping']['country_code'] = strtoupper( $order->get_shipping_country() );
                }
                if ( ! empty( $order->get_shipping_first_name() ) ) {
                    $customer_info['shipping']['first_name'] = $this->s40( $order->get_shipping_first_name(), 50 );
                }
                if ( ! empty( $order->get_shipping_last_name() ) ) {
                    $customer_info['shipping']['last_name'] = $this->s40( $order->get_shipping_last_name(), 50 );
                }
                if ( ! empty( $order->get_shipping_postcode() ) ) {
                    $customer_info['shipping']['postal_code'] = $this->s40( $order->get_shipping_postcode(), 10 );
                }
                if ( ! empty( $order->get_shipping_state() ) ) {
                    $customer_info['shipping']['state'] = $this->s40( $order->get_shipping_state(), 50 );
                }
                if ( ! empty( $order->get_shipping_address_2() ) ) {
                    $customer_info['shipping']['address_2'] = $this->s40( $order->get_shipping_address_2() );
                }
            }

            if ( ! empty( $customer_info['phone_number'] ) ) {
                $this->sanitizeValue( $customer_info['phone_number'], '' );
            }

            if ( ! empty( $customer_info['email_address'] ) ) {
                $this->sanitizeValue( $customer_info['email_address'], '' );
            }

            if ( ! empty( $customer_info['company_name'] ) ) {
                $this->sanitizeValue( $customer_info['company_name'], ',' );
            }

            $this->sanitizeCustomerInfo( $customer_info['billing'] );
            $this->sanitizeCustomerInfo( $customer_info['shipping'] );

            $result = $this->client->postCustomer( $customer_info );

            return $result['id'];
        } catch ( \VerifoneEcomAPI\ApiWrapper\Http\HttpException $exception ) {
            $this->logger->debug( 'There was an error in the API response: ' . PHP_EOL .
                                  'Sent request: ' . json_encode( $customer_info ) . PHP_EOL .
                                  'Received response: ' . $exception->getMessage() . PHP_EOL .
                                  'To endpoint: ' . $this->api_url_mapper->postCustomerUrl()
            );

            return null;
        }
    }

    public function verifone_checkout_capture( \WC_Order $order ) {
        $authType      = 'FINAL_AUTH'; //default value
        $manualCapture = 'NO';
        foreach ( $order->get_meta_data() as $val ) {
            $arr = $val->get_data();
            if ( $arr['key'] === '__verifone_order_transaction_type' ) {
                $authType = $arr['value'];
            }
            if ( $arr['key'] === '__verifone_order_capture_manual' ) {
                $manualCapture = $arr['value'];
            }
        }

        //display buttons only when the order is in PENDING and the MANUAL CAPTURE is set (PRE or FINAL auth)
        if ( ! self::$REFUND_BUTTON_RENDERED && $manualCapture === 'yes' && in_array( $order->get_status(), [ 'pending', 'processing' ] ) ) {
            self::$REFUND_BUTTON_RENDERED = true;
            $url                          = get_site_url() . '/?wc-api=manual_capture_payment&order_id=' . $order->get_id();
            include( plugin_dir_path( __FILE__ ) . '../templates/html-order-capture.php' );
        }

        add_action( 'woocommerce_order_item_add_action_buttons', array( $this, 'add_capture_button' ) );

        add_action( 'wp_ajax_wc_' . $this->id . '_capture_charge', array( $this, 'ajax_process_capture' ) );
    }

    /**
     *
     * @throws Exception
     */
    public function manual_capture_payment() {
        ob_start();

        $params   = $_POST;
        $order_id = $params['order_id'];
        $order    = wc_get_order( $order_id );
        $reason   = $params['reason'];
        $amount   = (float) $params['amount'];

        if ( ! $order ) {
            wp_send_json( [ 'result' => 'failure', 'messages' => 'Missing or invalid order ID.' ], 402 );
        }
        $total = $order->get_total();
        if ( ! current_user_can( 'edit_shop_order', $order_id ) ) {
            wp_send_json( [ 'result' => 'failure', 'messages' => 'Unauthorized' ], 401 );
        }
        if ( $amount > $total ) {
            wp_send_json( [ 'result' => 'failure', 'messages' => 'Forbidden - capture amount cannot be greater than the order total!' ], 403 );
        }

        if ( $amount ) { //full or partial capture
            $this->voidOrCapture( $order_id, 'capture', $this->calculateAmount( $amount ) );
        } else {  // full capture only
            $amount = $total;
            $this->voidOrCapture( $order_id );
        }
        $note = 'Order amount captured: ' . $amount . ' ' . $order->get_currency() . '. Refund is not available within the first 24-48 hours.';
        if ( $reason ) {
            $note .= ' *Reason: ' . $reason;
        }
        $order->update_status( 'completed', $note );

        //update order's total with the paid amount
        $order->update_meta_data( '_order_total', $amount );

        wp_send_json( [ 'result' => 'success', 'message' => $note ], 200 );
    }

    /**
     * @param $order_id
     */
    public function cancel_payment( $order_id ) {
        $this->voidOrCapture( $order_id, 'void' );
    }

    /**
     * @param $order_id
     */
    public function complete_payment( $order_id ) {
        $this->voidOrCapture( $order_id );
    }

    protected function logOrderComment($status, $transactionId, $order) {
        $note = __("Notification event: " . $status);
        $order->add_order_note($note, false, false);

        $note = __("Transaction ID: " . ((string)$transactionId));
        $order->add_order_note($note, false, false);
    }

    protected function stringStartsWith( $string, $needle ) {
        $len = strlen( $needle );

        return ( substr( $string, 0, $len ) === $needle );
    }

    protected function orderCheckAndLogEvent( WC_Order &$order, string $eventId ): bool {
        $existingEvents = json_decode( $order->get_meta( VERIFONE_META_EVENT_KEY ) ?: "[]", true );
        if ( in_array( $eventId, $existingEvents ) ) {
            return true;
        }

        $existingEvents[] = $eventId;
        $order->update_meta_data( VERIFONE_META_EVENT_KEY, json_encode( $existingEvents ) );
        $order->save_meta_data();

        return false;
    }

    public function noteExists( $orderId, $string ) {
        global $wpdb;
        $table   = "{$wpdb->prefix}comments";
        $results = $wpdb->get_results( 'SELECT * FROM ' . $table . ' WHERE comment_post_ID = ' . $orderId . ' AND comment_content  LIKE "%' . $string . '%"' );

        return (bool) count( $results );
    }

    /**Check if the WooCommerce Blocks plugin is active
     * @return bool
     */
    function isWoocommerceBlocksAvailable():bool {
        return defined("WC_BLOCKS_IS_FEATURE_PLUGIN");
    }
}
